==> webkit2gtk_2.44.2.orig.tar.xz: webkitgtk-2.44.2/Source/JavaScriptCore/runtime/KeyAtomStringCacheInlines.h <==
/*
 * Copyright (C) 2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "Identifier.h"
#include "KeyAtomStringCache.h"
#include "SmallStrings.h"
#include "VM.h"

namespace JSC {

template<typename Buffer, typename Func>
ALWAYS_INLINE JSString* KeyAtomStringCache::make(VM& vm, Buffer& buffer, const Func& func)
{
    if (!buffer.length)
        return jsEmptyString(vm);

    if (buffer.length == 1) {
        auto firstCharacter = buffer.characters[0];
        if (firstCharacter <= maxSingleCharacterString)
            return vm.smallStrings.singleCharacterString(firstCharacter);
    }

    ASSERT(buffer.length <= maxStringLengthForCache);
    auto& slot = m_cache[buffer.hash % capacity];
    if (slot) {
        auto* impl = slot->tryGetValueImpl();
        if (impl->hash() == buffer.hash && equal(impl, buffer.characters, buffer.length))
            return slot;
    }

    JSString* result = func(vm, buffer);
    if (LIKELY(result))
        slot = result;
    return result;
}

} // namespace JSC

==> webkit2gtk_2.51.90.orig.tar.xz: webkitgtk-2.51.90/Source/JavaScriptCore/runtime/KeyAtomStringCacheInlines.h <==
/*
 * Copyright (C) 2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <JavaScriptCore/Identifier.h>
#include <JavaScriptCore/JSString.h>
#include <JavaScriptCore/KeyAtomStringCache.h>
#include <JavaScriptCore/SmallStrings.h>
#include <JavaScriptCore/VM.h>

namespace JSC {

template<typename Buffer, typename Func>
ALWAYS_INLINE JSString* KeyAtomStringCache::make(VM& vm, Buffer& buffer, const Func& func)
{
    if (buffer.characters.empty())
        return jsEmptyString(vm);

    if (buffer.characters.size() == 1) {
        auto firstCharacter = buffer.characters[0];
        if (firstCharacter <= maxSingleCharacterString)
            return vm.smallStrings.singleCharacterString(firstCharacter);
    }

    ASSERT(buffer.characters.size() <= maxStringLengthForCache);
    auto& slot = m_cache[buffer.hash % capacity];
    if (slot) {
        auto* impl = slot->tryGetValueImpl();
        if (impl->hash() == buffer.hash && equal(impl, buffer.characters))
            return slot;
    }

    JSString* result = func(vm, buffer);
    if (result) [[likely]]
        slot = result;
    return result;
}

} // namespace JSC
